@@ -6,7 +6,7 @@ Check github at ...
 
 __title__ = 'DiscordGateway'
 __author__ = 'zYoshaa'
-__version__ = '1.0.5'
+__version__ = '2.0.0'
 
 from .api import *
 from .constants import *
@@ -16,6 +16,10 @@ from .gatewaybot import *
 from .gateway_handler import *
 from .gateway import *
 from .embed import *
+from .request_manager import *
+from .presence import *
+from.guild_objects import *
+from .exceptions import *
 import logging as log
 
-log.basicConfig(encoding='utf-8', level=log.ERROR)
\ No newline at end of file
+log.basicConfig(encoding='utf-8', level=log.ERROR)
