@@ -100,7 +100,7 @@ class GatewayCon:
             await self.send(ping)
 
     async def handle_message(self, msg: GatewayMessage) -> None:
-        ...  # Extendable
+        ...  # Override
 
     async def send(self, msg: dict) -> None:  # Send data to websocket
         try:
