@@ -1,27 +1,28 @@
-from typing import Optional, Callable, Any, Union
+from typing import Optional, Callable, Any, Union, AsyncIterator, Iterator
 
 from requests import Response
 
 from discord_gateway.discord_objects import DiscordChannel, DiscordGuild
-from discord_gateway.discord_user import DiscordUser
+from discord_gateway.discord_user import DiscordUser, DiscordMember
 from discord_gateway.gateway_handler import Gateway
 from discord_gateway.gateway import GatewayMessage
 from discord_gateway.api import HttpClient
 from types import SimpleNamespace
-from discord_gateway.constants import Intents
-
+from discord_gateway.constants import Intents, Status
+from discord_gateway.presence import DiscordActivity
 
 __all__ = (
     'GatewayBot',
+    'ClientUser'
 )
 
 
 class GatewayBot(Gateway):
 
-    def __init__(self, *, intents: int = Intents.All):
-        super().__init__(intents=intents.value)
-        self.api = None
-        self.user: Optional[SimpleNamespace] = None
+    def __init__(self, *, intents: int = Intents.All, allow_dms: bool = True) -> None:
+        super().__init__(intents=intents.value, allow_dms=allow_dms)
+        self.api: Optional[HttpClient] = None
+        self.user: Optional[ClientUser] = None
         self.add_listener(self.default_ready, event='ready', type='default')
 
     async def default_ready(self, msg: GatewayMessage) -> None:
@@ -31,10 +32,9 @@ class GatewayBot(Gateway):
         :param msg: :class:`GatewayMessage`
             The data received from the identify response
         """
+
         self.api = HttpClient(self._token, bot=self.is_bot)
-        self.user = SimpleNamespace(**msg.data.user)
-        self.user.guilds = msg.data.__dict__.get('guilds')  # Guilds IDs only
-        self.user.tag = f'{self.user.username}#{self.user.discriminator}'
+        self.user = ClientUser(msg, self.allow_dms, client=self.api)
 
     def event(self, func: Callable) -> Callable[..., Any]:
         """
@@ -48,13 +48,32 @@ class GatewayBot(Gateway):
             'on_') else func.__name__)
         return func
 
+    @property
+    def dms(self) -> Union[Response, dict]:
+        """
+        Get open dms of bot\n
+        :return: :class:`Union[Response, dict]`
+            Returns the open dms
+        """
+        return self.api.requests(method='GET', path='/users/@me/channels')
+
+    @property
+    def guilds(self) -> Iterator[DiscordGuild]:
+        """
+        Get current user guilds\n
+        :return:
+            Iterator of the guilds
+        """
+        for guild in self.api.requests(method='GET', path='/users/@me/guilds'):
+            yield DiscordGuild(guild.get('id'), api=self.api)
+
     async def get_channel(self, channel_id: str) -> DiscordChannel:
         """
         Get channel by his id\n
         :param channel_id: :class:`str`
             The channel id
         :return: :class:`DiscordChannel`
-            Returns the channel
+            The channel
         """
         return DiscordChannel(channel_id, api=self.api)
 
@@ -68,14 +87,6 @@ class GatewayBot(Gateway):
         """
         return DiscordGuild(guild_id, api=self.api)
 
-    async def dms(self) -> Union[Response, dict]:
-        """
-        Get open dms of bot\n
-        :return: :class:`Union[Response, dict]`
-            Returns the open dms
-        """
-        return self.api.requests(method='GET', path='/users/@me/channels')
-
     async def get_user(self, user_id: str) -> DiscordUser:
         """
         Get user by his id\n
@@ -91,6 +102,20 @@ class GatewayBot(Gateway):
         return DiscordUser(self.api.requests(method='POST', path=f'/users/@me/channels', json={"recipients": [user_id]},
                                              headers=headers), self.api)
 
+    async def change_presence(self, activity: Optional[DiscordActivity] = None,
+                              status: Union[Status, str] = Status.Online) -> None:
+        """
+        Change self presence\n
+        :param activity:
+            The activity to show
+        :type activity: Optional[DiscordActivity]
+        :param status:
+            The status of bot ex. idle
+        :type status: Union[Status, str]
+        """
+        payload = await self.api.manager.get_presence(activity, status)
+        await self.send(payload)
+
     async def join(self, server_invite: str) -> Union[Response, dict]:
         """
         Join a guild by invite {SelfBot only}\n
@@ -122,3 +147,53 @@ class GatewayBot(Gateway):
             'content-type': 'application/json'
         }
         return self.api.requests(method='POST', path=f'/users/@me/channels', json={'recipients': ids}, headers=headers)
+
+    async def members_from_guilds(self, guild_id: str) -> AsyncIterator[DiscordMember]:
+        """
+        Get possible members from a guild
+        :param guild_id:
+        :return:
+        """
+        if guild_id not in [guild.get('id') for guild in self.user.guilds]:
+            raise AttributeError('Guild not found')
+        for member in list(filter(lambda guild: guild.get('id') == guild_id, self.user.guilds))[0]['members']:
+            yield DiscordMember(member, member.get('user'), guild_id, client=self.api)
+
+
+class ClientUser(SimpleNamespace):
+    __slots__ = 'api', 'id', 'username', 'discriminator', 'bot', 'allow_dms', 'guilds', 'tag'
+
+    def __init__(self, msg: GatewayMessage, allow_dms: bool, client: HttpClient):
+        super(ClientUser, self).__init__(**msg.data.user)
+        self.api = client
+        self.id: str = msg.data.user.get('id')
+        self.username: str = msg.data.user.get('username')
+        self.discriminator: str = msg.data.user.get('discriminator')
+        self.avatar: Any = msg.data.user.get('avatar')
+        self.bot: bool = msg.data.user.get('bot', False)
+        self.allow_dms = allow_dms  # Check if dms are allowed in config
+        self.tag = f'{self.username}#{self.discriminator}'
+
+    async def edit(
+            self,
+            username: Optional[str] = None,
+            *,
+            avatar: Any = None
+    ) -> Union[Response, dict]:
+        """
+        Edit a user profile\n
+        :param username:
+            New username
+        :type username: Optional[str]
+        :param avatar:
+            New pfp (profile picture/avatar)
+        :type avatar: Any
+        :return
+            The new updated user
+        """
+        payload: dict[str, Any] = {'username': (username or self.username)}
+
+        if avatar:
+            payload[avatar] = avatar
+
+        return await self.api.manager.edit_bot_user(payload=payload)
